;===================================================================
;                   RANDOM.ASM
;      r a n d o m   n u m b e r   g e n e r a t o r
;
; GENERATES PSEUDO-RANDOM INTEGERS IN THE RANGE LOWER TO UPPER
; INPUT:  TWO STACK PARAMETERS - LOWER AND UPPER ENDS OF RANGE
; OUTPUT: AX-REG CONTAINS RANDOM INTEGER
; CALLING SEQUENCE:     PUSH    <LOWER END OF RANGE>
;                       PUSH    <UPPER END OF RANGE>
;                       CALL    RANDOM

;as a check to see if it is working when the seed is 555H, the first few numbers should be 832, 6809, 8658, and 1206

;===================================================================

         .MODEL  LARGE       ;, BASIC
;        .586                ;we are using 16-bit code i commented this line out
;        .STACK  256         ;the stack should be defined in main.asm - not here

;===================================================================

        PUBLIC  SEED
        PUBLIC  RESEED
        PUBLIC  NRANDOM

;===================================================================

; D A T A   S E G M E N T   D E F I N I T I O N

        .FARDATA

SEED       DW      ?                   ;SEED FOR RANDOM NUMBER GEN.
MULTIPLIER DW      25173               ;MULTIPLIER AND
ADDEND     DW      13849               ;ADDEND FOR MIXED
                                       ;LINEAR CONGRUENTIAL METHOD

;===================================================================

; C O D E   S E G M E N T   D E F I N I T I O N

        .CODE   
        ASSUME  DS:NOTHING,ES:FAR_DATA

;-------------------------------------------------------------------

RESEED  PROC    FAR PUBLIC USES AX CX DX ES

        MOV     AX,FAR_DATA            ;SET ES-REGISTER TO POINT
        MOV     ES,AX                  ;TO FAR DATA SEGMENT

                                       ;SAVE REGISTERS (USES LIST)
        PUSHF                          ;SAVE FLAGS - neccessary ?
        mov     dx,5555h
        or      bl,bl
        jnz     NOTZERO

        MOV     AH,0                   ;     SEED = LOWER HALF OF
        INT     1AH                    ;            TIME OF DAY CLOCK

NOTZERO:
        MOV     SEED,dx
        POPF                           ;RESTORE FLAGS
                                       ;RESTORE REGISTERS (ASSEMBLER
                                       ;GENERATES INSTRUCTIONS TO
                                       ;RESTORE REGISTERS - USES)
        RET                            ;RETURN

RESEED  ENDP                     

;-------------------------------------------------------------------

NRANDOM PROC    FAR PUBLIC

                                       ;FUNCTION RANDOM(LOWER,UPPER)

; the return segment is [BP+12]
; the return address is [BP+10]

        PUSH    CX                     ;( [BP+8] )
        PUSH    DX                     ;( [BP+6] )
        PUSH    ES                     ;( [BP+4] )
        PUSHF                          ;SAVE FLAGS ( [BP+2] )
        PUSH    BP                     ;( [BP+0] )
        MOV     BP,SP
        MOV     AX,FAR_DATA            ;SET ES-REGISTER TO POINT
        MOV     ES,AX                  ;TO FAR DATA SEGMENT

        MOV     AX,SEED                ;X = SEED * MULTIPLIER mod 65536
        MUL     MULTIPLIER
        ADD     AX,ADDEND              ;SEED = (X + ADDEND) mod 65536
        MOV     SEED,AX
        MOV     CX,[BP+16]             ;UPPER - RANGE = UPPER - LOWER + 1
        SUB     CX,[BP+14]             ;LOWER
        INC     CX
        MUL     CX                     ;RANDOM = (SEED*RANGE)/65536
        ADD     DX,[BP+14]             ;                    + LOWER
        MOV     AX,DX
        POP     BP
        POPF                           ;RESTORE FLAGS
        POP     ES                     ;RESTORE REGISTERS
        POP     DX
        POP     CX
        RET     4                      ;RETURN (RANDOM)

NRANDOM ENDP                           ;END NRANDOM

;-------------------------------------------------------------------

        END   ; RESEED   no entry point is required for linked modules

